%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Technical Note TN162
% Parameters for 'Finite Control Set Model Predictive Control
% of voltage-controlled inverter'
% Copyright imperix ltd, 2024
% dev@imperix.ch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clearvars -except variant_generation variant_simulation SIMULINK_MODE
clc

% Main parameters

Fs = 100e3;                 % Sampling frequency [Hz]
Ts = 1/Fs;                  % Sampling period [s]

% Plant parameters

Vdc = 800;

Cbus = 3e-3;                % Half-bus capacitors [F]
Rbus = 1e-2;                % Parasitic resistances of Cbus [Ohm]

L1 = 2.2e-3;                % LCL filter inductances (inverter side) [H]
R1 = 22e-3;                 % Parasitic resistances of L1 [Ohm]
L2 = 2.2e-3;                % LCL filter inductances (load side) [H]
R2 = 22e-3;                 % Parasitic resistances of L2 [Ohm]

Cf = 10e-6;                 % Main capacitors [F]
R = 1;                      % Damping resistors [Ohm]
Cfb = 1e-6;                 % Feedback-path capacitor [F]
CEMC = 3.3e-6;              % EMC-filter capacitors [F]

Rload = 30;                 % Load [Ohm]

% A and B matrices

Ceq = Cf+CEMC;

A = [-R1/L1 -1/L1; 1/Ceq 0];
B = [1/L1 0; 0 -1/Ceq];

Ceq = 1/(3/Cfb+1/Cf);
    
A0 = [-R1/L1 -1/L1; 1/Ceq 0];
B0 = [1/L1 0; 0 -1/Ceq];

% Exact discretization of A and B matrices

Ad = expm(A*Ts);
Bd = (integral(@(Ts) expm(A.*Ts),0,Ts, 'ArrayValued', true))*B;

Ad0 = expm(A0*Ts);
Bd0 = (integral(@(Ts) expm(A0.*Ts),0,Ts, 'ArrayValued', true))*B0;