

/*
 * Include Files
 *
 */
#if defined(MATLAB_MEX_FILE)
#include "tmwtypes.h"
#include "simstruc_types.h"
#else
#include "rtwtypes.h"
#endif

/* %%%-SFUNWIZ_wrapper_includes_Changes_BEGIN --- EDIT HERE TO _END */
float simulate_state(
    const float sw[3],
    float Vdc,
    float IO[2],
    float Vref[3],
    float ii_k1[3],
    float vc_k1[3],
    float Ad[2][2],
    float Bd[2][2],
    float Ad0[2][2],
    float Bd0[2][2],
    int mode) {

    float vi_k1[3];
    float ii_k2[3];
    float vc_k2[3];
    float iiref_k2[3];

    float cost = 0;

    // 2nd-step calculations for the alpha/beta-components
    for(int i=0;i<2;i++) {
        vi_k1[i] = Vdc*sw[i];

        ii_k2[i] = Ad[0][0]*ii_k1[i] + Ad[0][1]*vc_k1[i] + Bd[0][0]*vi_k1[i] + Bd[0][1]*IO[i];
        vc_k2[i] = Ad[1][0]*ii_k1[i] + Ad[1][1]*vc_k1[i] + Bd[1][0]*vi_k1[i] + Bd[1][1]*IO[i];

        iiref_k2[i] = (Vref[i] - Ad[1][1]*vc_k2[i] - Bd[1][0]*vi_k1[i] - Bd[1][1]*IO[i]) / Ad[1][0];

        cost += (iiref_k2[i]-ii_k2[i])*(iiref_k2[i]-ii_k2[i]);
    }

    // 2nd-step calculations for the 0-component
    if (mode==1) {
        vi_k1[2] = Vdc*sw[2]-Vdc/2;

        ii_k2[2] = Ad0[0][0]*ii_k1[2] + Ad0[0][1]*vc_k1[2] + Bd0[0][0]*vi_k1[2] + Bd0[0][1]*IO[2];
    
        cost += 50*ii_k2[2]*ii_k2[2];
    }

    return cost;
}
/* %%%-SFUNWIZ_wrapper_includes_Changes_END --- EDIT HERE TO _BEGIN */
#define u_width 1
#define y_width 1
/*
 * Create external references here.  
 *
 */
/* %%%-SFUNWIZ_wrapper_externs_Changes_BEGIN --- EDIT HERE TO _END */
/* extern double func(double a); */
/* %%%-SFUNWIZ_wrapper_externs_Changes_END --- EDIT HERE TO _BEGIN */

/*
 * Output functions
 *
 */
void MPC_Outputs_wrapper(const real32_T *Vdc_ptr,
			const real32_T *ii_ptr,
			const real32_T *vc_ptr,
			const real32_T *io_ptr,
			const real32_T *vref_ptr,
			const real32_T *Ad_ptr,
			const real32_T *Bd_ptr,
			const real32_T *Ad0_ptr,
			const real32_T *Bd0_ptr,
			const int32_T *mode_ptr,
			uint32_T *out)
{
/* %%%-SFUNWIZ_wrapper_Outputs_Changes_BEGIN --- EDIT HERE TO _END */
float Vdc = *Vdc_ptr;
float *IM = (float*)ii_ptr;
float *V = (float*)vc_ptr;
float *IO = (float*)io_ptr;
float *Vref = (float*)vref_ptr;
float Ad[2][2] = {{Ad_ptr[0], Ad_ptr[2]},{Ad_ptr[1], Ad_ptr[3]}};
float Bd[2][2] = {{Bd_ptr[0], Bd_ptr[2]},{Bd_ptr[1], Bd_ptr[3]}};
float Ad0[2][2] = {{Ad0_ptr[0], Ad0_ptr[2]},{Ad0_ptr[1], Ad0_ptr[3]}};
float Bd0[2][2] = {{Bd0_ptr[0], Bd0_ptr[2]},{Bd0_ptr[1], Bd0_ptr[3]}};
int mode = *mode_ptr;

// constants
const float _1_3 = 0.33333333;
const float _2_3 = 0.66666667;
const float _sqrt3_3 = 0.57735027;

// possible switching states
// leg_a = LSB, leg_c = MSB
const unsigned int states[8] = {0b000, 0b001, 0b011, 0b010, 0b110, 0b100, 0b101, 0b111};

// normalized alpha/beta/0-components at the output of the inverter
// corresponding to the switching states of states[7]
const float sw[8][3] =
    {{0, 0, 0}, {_2_3, 0, _1_3}, {_1_3, _sqrt3_3, _2_3}, {-_1_3, _sqrt3_3, _1_3}, {-_2_3, 0, _2_3}, {-_1_3, -_sqrt3_3, _1_3}, {_1_3, -_sqrt3_3, _2_3}, {0, 0, 1}};

// current state
static int x_k = 0;

// start of 1st step
float vi_k[3];
float ii_k1[3];
float vc_k1[3];

// 1st-step calculations for the alpha/beta-components
for(int i=0;i<2;i++) {
    vi_k[i] = Vdc*sw[x_k][i];
}

for(int i=0;i<2;i++) {
    ii_k1[i] = Ad[0][0]*IM[i] + Ad[0][1]*V[i] + Bd[0][0]*vi_k[i] + Bd[0][1]*IO[i];
    vc_k1[i] = Ad[1][0]*IM[i] + Ad[1][1]*V[i] + Bd[1][0]*vi_k[i] + Bd[1][1]*IO[i];
}

// 1st-step calculations for the 0-component
if (mode==1) {
    vi_k[2] = Vdc*sw[x_k][2]-Vdc/2;

    ii_k1[2] = Ad0[0][0]*IM[2] + Ad0[0][1]*V[2] + Bd0[0][0]*vi_k[2] + Bd0[0][1]*IO[2];
    vc_k1[2] = Ad0[1][0]*IM[2] + Ad0[1][1]*V[2] + Bd0[1][0]*vi_k[2] + Bd0[1][1]*IO[2];
}

// start of 2nd step
float g;
float min_g = 1e18;
int min_ind = 0;

// simulation of all possible states and identification of the optimal one
for (int i=0; i<8; i++) {
    g = simulate_state(sw[i], Vdc, IO, Vref, ii_k1, vc_k1, Ad, Bd, Ad0, Bd0, mode);

    if (g < min_g) {
        min_g = g;
        min_ind = i;
    }
}

// update of state and out
x_k = min_ind;
*out = states[x_k];
/* %%%-SFUNWIZ_wrapper_Outputs_Changes_END --- EDIT HERE TO _BEGIN */
}
